/**
 * 在chanjet平台上实现的具体预处理类 , 需要监听cordova.deviceready事件
 *
 * Created by tonyjiang on 16/3/17.
 */

import Mutants from './Mutants'
import AndroidRestore from './AndroidRestore'
import NativeConfig from './native-config';


class MutantsForChanjet extends Mutants{

    constructor(){
        super();
    }


    _ready(){
        document.addEventListener('deviceready' , this._deviceReadyHandler.bind(this));
        document.addEventListener('resume', this._deviceResumeHandler.bind(this));
        document.addEventListener('pause', this._deviceSuspendHandler.bind(this));

    }

    _deviceReadyHandler(){
        // 获取 native config 信息
        this._retrieveNativeConfig()
            .then(() => {
                //如果是安卓, 则需要查询restore状态及数据, 查询完成后, 在进行后续处理
                if(this.env.os == this.env.constant.os.android){
                    AndroidRestore.ready(this._readyCallback.bind(this));
                }else{
                    this._readyCallback();
                }
            })
            .catch(err => {
                console.error(err);
                this._readyCallback(err);
            })
    }

    _deviceResumeHandler(){
        this.emitter.emit(Mutants.EVENT_NAME.RESUME);
    }

    _deviceSuspendHandler(){
        this.emitter.emit(Mutants.EVENT_NAME.SUSPEND);
    }

    _readyCallback(error){
        this._isReady = true;
        this.emitter.emit(Mutants.EVENT_NAME.READY, error);
    }
    
    get isRestore(){
        return AndroidRestore.isRestore;
    }

    get restorePluginData(){
        return AndroidRestore.restorePluginData;
    }

    _retrieveNativeConfig() {
        return new Promise((resolve, reject) => {
            const success = (resp) => {
                try {
                    if (typeof(resp) === 'string') {
                        resp = JSON.parse(resp);
                    }

                    if (resp.data && resp.data.platformExtension && resp.data.platformExtension !== '') {
                        // 解析 config 对象
                        NativeConfig.parseConfig(resp);
                        resolve(NativeConfig.instance);
                    }
                    else {
                        reject('没有得到正确的 config 数据.');
                    }
                }
                catch (err) {
                    reject(err.toString());
                }
            };

            const error = (error) => {
                reject(error);
            };

            if (window.cordova && window.cordova.exec) {
                // 有 cordova , 调用 native 插件读取 config 信息
                window.cordova.exec(success, error, 'Config', 'getConfig', []);
            }
            else {
                reject('没有 cordova 对象, 请确认 cordova 初始化是否正确.');
            }
        })
    }
}

export default MutantsForChanjet