/**
 * Plugin管理对象
 * 提供插件统一调用的地方, 所有的插件都会被加载, 并且封装一个get方法来获取实例, 确保每一个插件都是单例.
 *
 * PluginMap中定义了对外公开的插件名 和 对应的插件Class. 并且保存了插件的实例对象.
 *      插件对外公开的插件名都是本身类名去掉后缀Plugin , 然后再转成驼峰的结果.
 *      只有phone一个例外. 后期考虑将PhoneCallPlugin改名为PhonePlugin.
 *
 * Created by TonyJiang on 16/4/28.
 */

import {Plugin , PluginMocker} from 'chanjet-plugin-base';

import AuthPlugin from 'chanjet-plugin-auth';
import ChoosePhotoPlugin from 'chanjet-plugin-choose-photo';
import TakePhotoPlugin from 'chanjet-plugin-take-photo';
import GeoPlugin from 'chanjet-plugin-geo';
import PhoneCallPlugin from 'chanjet-plugin-phone-call';
import PreviewPlugin from 'chanjet-plugin-preview';
import RestoreStatusPlugin from 'chanjet-plugin-restore-status';
import SharePlugin from 'chanjet-plugin-share';
import SMSPlugin from 'chanjet-plugin-sms';
import SpeechRecoPlugin from 'chanjet-plugin-speech-reco';
import UploadPlugin from 'chanjet-plugin-upload';
import NavigationPlugin from 'chanjet-plugin-navigation';
import AlertPlugin from 'chanjet-plugin-alert';
import Member from 'chanjet-plugin-member';
import DatePicker from 'chanjet-plugin-datepicker';
// import ScanCardPlugin from '../../chanjet-plugins/chanjet-plugin-scan-card/src';


// import ChoosePhotoPlugin from '../../chanjet-plugins/chanjet-plugin-choose-photo/src';
// import PreviewPlugin from '../../chanjet-plugins/chanjet-plugin-preview/src';
// import SpeechRecoPlugin from '../../chanjet-plugins/chanjet-plugin-speech-reco/src';
// import SharePlugin from '../../chanjet-plugins/chanjet-plugin-share/src';

const PluginMap = {
    //企业信息
    auth : {
        class : AuthPlugin,
        instance : null
    },

    //选照片
    choosePhoto :{
        class : ChoosePhotoPlugin,
        instance : null
    },
    //拍照
    takePhoto : {
        class : TakePhotoPlugin,
        instance : null
    },
    //预览
    preview : {
        class : PreviewPlugin,
        instance : null
    },
    //上传
    upload : {
        class : UploadPlugin,
        instance : null
    },

    //定位
    geo : {
        class : GeoPlugin,
        instance : null
    },

    //打电话
    phone : {
        class : PhoneCallPlugin,
        instance : null
    },

    //短信
    sms : {
        class : SMSPlugin,
        instance : null
    },

    //分享
    share : {
        class : SharePlugin,
        instance : null
    },

    //语音识别
    speechReco : {
        class : SpeechRecoPlugin,
        instance : null
    },

    //android restore状态查询
    restoreStatus : {
        class : RestoreStatusPlugin,
        instance : null
    },

    alert: {
        class: AlertPlugin,
        instance: null
    },

    navigation : {
        class : NavigationPlugin,
        instance : null
    },

    member: {
        class: Member,
        instance: null
    },

    datepicker: {
        class: DatePicker,
        instance: null
    }

    // //名片扫描
    // scanCard : {
    //     class : ScanCardPlugin,
    //     instance : null
    // }
};


class PluginManager {

    constructor () {

        //遍历所有插件,并提供getter方法
        for(let name in PluginMap){
            Object.defineProperty(this , name , {
                get : function(){
                    if(!PluginMap[name].instance){
                        PluginMap[name].instance = new PluginMap[name].class();
                    }
                    return PluginMap[name].instance;
                }
            })
        }


    }


    /**
     * 设置mock数据
     * @param mockData
     */
    setMockData(mockData){
        PluginMocker.data = mockData;
    }



}

const instance = new PluginManager;

export default instance;