/**
 * Created by tonyjiang on 16/3/17.
 */
import {jsdom} from 'jsdom';
import 'should';
import chai from 'chai';
import sinon from 'sinon';


import env from 'chanjet-env-check';

import MutantsChanjetImpl from '../src/MutantsChanjetImpl';
import PluginManager from '../src/PluginManager';

const assert = chai.assert;
const readyCallbackCount = 3;

describe("MutantsChanjetImpl单元测试" , function(){

    beforeEach(function(){
        //保证每次的document都是一个新的
        global.document = jsdom('<!doctype html><html><body></body></html>');
        window.cordova = {
            exec: sinon.stub()
        }

        window.cordova.exec.callsArgWithAsync(
            0,
            JSON.stringify(
                {
                    data: {
                        entId: 'entId',
                        platformExtension: JSON.stringify({
                            orgAccount: 'orgAccount'
                        })
                    },
                }
            )
        );
    });

    afterEach(function () {
        window.cordova = undefined;
    })

    describe("初始化" , function(){

        it("应该监听在document对象上监听deviceready, resume, pause事件" , function(){

            let instance = new MutantsChanjetImpl();

            let addEventListenerSpy = sinon.spy(document , 'addEventListener');
            let _readySpy = sinon.spy(instance , '_ready');


            instance.init();

            /**
             * _ready方法应该被调用一次,应该监听document的deviceready事件
             */
            assert(_readySpy.calledOnce);
            assert(addEventListenerSpy.callCount.should.equal(3));
            assert(addEventListenerSpy.args[0][0].should.equal('deviceready'));
            assert(addEventListenerSpy.args[1][0].should.equal('resume'));
            assert(addEventListenerSpy.args[2][0].should.equal('pause'));

            addEventListenerSpy.restore();
            _readySpy.restore();

        });

        it("document派发deviceready事件后,mutants._isReady应该等于true" , function(done){

            let instance = new MutantsChanjetImpl();

            instance.init();

            instance.ready((error) => {
                assert(instance._isReady.should.equal(true));
                done();
            });

            document.dispatchEvent(new Event('deviceready'));
        });



    });

    describe("ready" , function(){

        it("添加"+readyCallbackCount+"(0-10随机数)个ready回调, Mutants的回调队列长度等于"+readyCallbackCount , function(){
            let instance = new MutantsChanjetImpl();
            instance.init();

            for(let i = 0 ; i < readyCallbackCount ; i ++){
                instance.ready(sinon.spy());
            }

            assert(instance.emitter.listeners('ready').length.should.equal(readyCallbackCount));
            // assert(instance._readyCallbackQueue.length.should.equal(readyCallbackCount));

        });

        it(`添加 ${readyCallbackCount} 个ready回调, document触发 deviceready后,每一个回调都被执行` , function(done){
            let instance = new MutantsChanjetImpl();
            instance.init();

            let readyCount = 0;

            for(let i = 0 ; i < readyCallbackCount ; i ++){
                instance.ready(() => {
                    readyCount++;
                    if (readyCount == readyCallbackCount) {
                        done();
                    }
                });
            }

            document.dispatchEvent(new Event('deviceready'));
        });

        it("添加3个ready回调,document 触发 deviceready 后,当其中一个抛出异常,不影响其他回调执行" , function(done){
            let instance = new MutantsChanjetImpl();
            instance.init();

            const readyCallbackCount = 3;
            let readyCount = 0;

            for(let i = 0 ; i < readyCallbackCount ; i ++){
                instance.ready(() => {
                    readyCount++;
                    if (readyCount == readyCallbackCount) {
                        done();
                    }

                    if (i == 1) {
                        console.log('请忽略下方的Error: test, 测试callback中抛出');
                        throw new Error('test')
                    }
                });
            }

            document.dispatchEvent(new Event('deviceready'));

        });

        it("当deviceready后,再调用ready方法,应该直接执行回调" , function(done){
            let instance = new MutantsChanjetImpl();
            instance.init();

            instance.ready(() => {
                let callbackSpy = sinon.spy();
                instance.ready(callbackSpy);
                assert(callbackSpy.calledOnce);
                done();
            });

            document.dispatchEvent(new Event('deviceready'));
        })

    });

    describe("环境检测" , () => {
        it("mutants.env返回环境检测对象" , () => {
            assert(env == new MutantsChanjetImpl().env );
        });

        it("mutants.constant返回环境检测的常量" , () => {
           assert(env.constant == new MutantsChanjetImpl().constant);
        });

    });

    describe('插件' , () => {

        it('mutants.plugin应该是PluginManager' , () => {
            assert(PluginManager == new MutantsChanjetImpl().plugin);
        });

    });


});